/*
* Copyright (c) 2014 Microsoft Open Technologies, Inc.
*
*    Licensed under the Apache License, Version 2.0 (the "License"); you may
*    not use this file except in compliance with the License. You may obtain
*    a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
*
*    THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR
*    CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
*    LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS
*    FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
*
*    See the Apache Version 2.0 License for specific language governing
*    permissions and limitations under the License.
*
*    Microsoft would like to thank the following companies for their review and
*    assistance with these files: Intel Corporation, Mellanox Technologies Ltd,
*    Dell Products, L.P., Facebook, Inc
*
* Module Name:
*
*    saibuffer.h
*
* Abstract:
*
*    This module defines SAI Buffer API
*
*/

#if !defined (__SAIBUFFER_H_)
#define __SAIBUFFER_H_

#include <saitypes.h>

/** \defgroup SAIBUFFER SAI - Buffer specific API definitions.
 *
 *  \{
 */

/**
 * @brief Enum defining ingress priority group attributes.
 */
typedef enum _sai_ingress_priority_group_attr_t
{
    SAI_INGRESS_PRIORITY_GROUP_ATTR_START, 
    /** buffer profile pointer [sai_object_id_t] */
    SAI_INGRESS_PRIORITY_GROUP_ATTR_BUFFER_PROFILE = SAI_INGRESS_PRIORITY_GROUP_ATTR_START,
    SAI_INGRESS_PRIORITY_GROUP_ATTR_END
} sai_ingress_priority_group_attr_t;

/**
* @brief Enum defining statistics for ingress priority group.
*/
typedef enum _sai_ingress_priority_group_stat_counter_t
{
    /** get/set rx packets count [uint64_t] */
    SAI_INGRESS_PRIORITY_GROUP_STAT_PACKETS = 0x00000000,

    /** get/set rx bytes count [uint64_t] */
    SAI_INGRESS_PRIORITY_GROUP_STAT_BYTES = 0x00000001,

    /** get current pg occupancy in bytes [uint64_t] */
    SAI_INGRESS_PRIORITY_GROUP_STAT_CURR_OCCUPANCY_BYTES = 0x00000002,

    /** get watermark pg occupancy in bytes [uint64_t] */
    SAI_INGRESS_PRIORITY_GROUP_STAT_WATERMARK_BYTES = 0x00000003,

    /** get current pg shared occupancy in bytes [uint64_t] */
    SAI_INGRESS_PRIORITY_GROUP_STAT_SHARED_CURR_OCCUPANCY_BYTES = 0x00000004,

    /** get watermark pg shared occupancy in bytes [uint64_t] */
    SAI_INGRESS_PRIORITY_GROUP_STAT_SHARED_WATERMARK_BYTES = 0x00000005,

    /** get current pg xoff room occupancy in bytes [uint64_t] */
    SAI_INGRESS_PRIORITY_GROUP_STAT_XOFF_ROOM_CURR_OCCUPANCY_BYTES = 0x00000006,

    /** get watermark pg xoff room occupancy in bytes [uint64_t] */
    SAI_INGRESS_PRIORITY_GROUP_STAT_XOFF_ROOM_WATERMARK_BYTES = 0x00000007,

    /** -- */
    /** Custom range base value */
    SAI_INGRESS_PRIORITY_GROUP_STAT_CUSTOM_RANGE_BASE = 0x10000000
} sai_ingress_priority_group_stat_counter_t;

/**
 * @brief Set ingress priority group attribute
 * @param[in] ingress_pg_id ingress priority group id
 * @param[in] attr attribute to set
 *
 * @return  SAI_STATUS_SUCCESS on success
 *           Failure status code on error
 */
typedef sai_status_t(*sai_set_ingress_priority_group_attr_fn)(
    _In_ sai_object_id_t ingress_pg_id,
    _In_ const sai_attribute_t *attr
    );

/**
 * @brief Get ingress priority group attributes
 * @param[in] ingress_pg_id ingress priority group id
 * @param[in] attr_count number of attributes
 * @param[inout] attr_list array of attributes
 *
 * @return  SAI_STATUS_SUCCESS on success
 *           Failure status code on error
 */
typedef sai_status_t(*sai_get_ingress_priority_group_attr_fn)(
    _In_ sai_object_id_t ingress_pg_id,
    _In_ uint32_t attr_count,
    _Inout_ sai_attribute_t *attr_list
    );

/**
* @brief   Get ingress priority group statistics counters.
*
* @param[in] ingress_pg_id ingress priority group id
* @param[in] counter_ids specifies the array of counter ids
* @param[in] number_of_counters number of counters in the array
* @param[out] counters array of resulting counter values.
*
* @return SAI_STATUS_SUCCESS on success
*         Failure status code on error
*/
typedef sai_status_t(*sai_get_ingress_priority_group_stats_fn)(
    _In_ sai_object_id_t ingress_pg_id,
    _In_ const sai_ingress_priority_group_stat_counter_t *counter_ids,
    _In_ uint32_t number_of_counters,
    _Out_ uint64_t* counters
    );

/**
* @brief   Clear ingress priority group statistics counters.
*
* @param[in] ingress_pg_id ingress priority group id
* @param[in] counter_ids specifies the array of counter ids
* @param[in] number_of_counters number of counters in the array
*
* @return SAI_STATUS_SUCCESS on success
*         Failure status code on error
*/
typedef sai_status_t(*sai_clear_ingress_priority_group_stats_fn)(
    _In_ sai_object_id_t ingress_pg_id,
    _In_ const sai_ingress_priority_group_stat_counter_t *counter_ids,
    _In_ uint32_t number_of_counters
    );

/**
 * @brief Enum defining buffer pool types.
 */
typedef enum _sai_buffer_pool_type_t
{
    /** Ingress buffer pool */
    SAI_BUFFER_POOL_INGRESS,

    /** Egress buffer pool */
    SAI_BUFFER_POOL_EGRESS,

} sai_buffer_pool_type_t;

/**
 * @brief Enum defining buffer threshold modes
 */
typedef enum _sai_buffer_threshold_mode_t
{
    /** static maximum */
    SAI_BUFFER_THRESHOLD_MODE_STATIC,

    /** dynamic maximum (relative) */
    SAI_BUFFER_THRESHOLD_MODE_DYNAMIC,

} sai_buffer_threshold_mode_t;

/**
 * @brief Enum defining buffer pool attributes.
 */
typedef enum _sai_buffer_pool_attr_t
{
    SAI_BUFFER_POOL_ATTR_START,
    /** READ-ONLY */

    /** shared buffer size in bytes [sai_uint32_t].
     * This is derived from substracting all reversed buffers of queue/port
     * from the total pool size. */
    SAI_BUFFER_POOL_ATTR_SHARED_SIZE = SAI_BUFFER_POOL_ATTR_START,

    /** READ-WRITE */

    /** buffer pool type [sai_buffer_pool_type_t]  (MANDATORY_ON_CREATE|CREATE_ONLY) */
    SAI_BUFFER_POOL_ATTR_TYPE,

    /** buffer pool size in bytes [sai_uint32_t] (MANDATORY_ON_CREATE|CREATE_AND_SET) */
    SAI_BUFFER_POOL_ATTR_SIZE,

    /** For the ingress buffer pool, we allocate a shared XOFF (GLOBAL HEADROOM) room for lossless traffic
    *  default to 0. */
    SAI_BUFFER_POOL_ATTR_XOFF_SIZE,

    /** shared threshold mode for the buffer pool [sai_buffer_threadhold_mode_t] (CREATE_ONLY)
     * (default to SAI_BUFFER_POOL_DYNAMIC_TH) */
    SAI_BUFFER_POOL_ATTR_TH_MODE,
    
    SAI_BUFFER_POOL_ATTR_END,

} sai_buffer_pool_attr_t;

/**
* @brief Enum defining statistics for buffer pool.
*/
typedef enum _sai_buffer_pool_stat_counter_t
{
    /** get current pool occupancy in bytes [uint64_t] */
    SAI_BUFFER_POOL_STAT_CURR_OCCUPANCY_BYTES = 0x00000000,

    /** get watermark pool occupancy in bytes [uint64_t] */
    SAI_BUFFER_POOL_STAT_WATERMARK_BYTES = 0x00000001,

    /** -- */
    /** Custom range base value */
    SAI_BUFFER_POOL_STAT_CUSTOM_RANGE_BASE = 0x10000000
} sai_buffer_pool_stat_counter_t;

/**
 * @brief Create buffer pool
 * @param[out] pool_id buffer pool id
 * @param[in] attr_count number of attributes
 * @param[in] attr_list array of attributes
 * @return SAI_STATUS_SUCCESS on success
 *           Failure status code on error
 */
typedef sai_status_t(*sai_create_buffer_pool_fn)(
    _Out_ sai_object_id_t* pool_id,
    _In_ uint32_t attr_count,
    _In_ const sai_attribute_t *attr_list
    );

/**
 * @brief Remove buffer pool
 * @param[in] pool_id buffer pool id
 * @return SAI_STATUS_SUCCESS on success
 *           Failure status code on error
 */
typedef sai_status_t(*sai_remove_buffer_pool_fn)(
    _In_ sai_object_id_t pool_id
    );

/**
 * @brief Set buffer pool attribute
 * @param[in] pool_id buffer pool id
 * @param[in] attr attribute
 * @return SAI_STATUS_SUCCESS on success
 *           Failure status code on error
 */
typedef sai_status_t(*sai_set_buffer_pool_attr_fn)(
    _In_ sai_object_id_t pool_id,
    _In_ const sai_attribute_t *attr
    );

/**
 * @brief Get buffer pool attributes
 * @param[in] pool_id buffer pool id
 * @param[in] attr_count number of attributes
 * @param[inout] attr_list array of attributes
 * @return SAI_STATUS_SUCCESS on success
 *           Failure status code on error
 */
typedef sai_status_t(*sai_get_buffer_pool_attr_fn)(
    _In_ sai_object_id_t pool_id,
    _In_ uint32_t attr_count,
    _Inout_ sai_attribute_t *attr_list
    );

/**
* @brief   Get buffer pool statistics counters.
*
* @param[in] pool_id buffer pool id
* @param[in] counter_ids specifies the array of counter ids
* @param[in] number_of_counters number of counters in the array
* @param[out] counters array of resulting counter values.
*
* @return SAI_STATUS_SUCCESS on success
*         Failure status code on error
*/
typedef sai_status_t(*sai_get_buffer_pool_stats_fn)(
    _In_ sai_object_id_t pool_id,
    _In_ const sai_buffer_pool_stat_counter_t *counter_ids,
    _In_ uint32_t number_of_counters,
    _Out_ uint64_t* counters
    );

/**
 * @brief Enum defining buffer profile attributes.
 */
typedef enum _sai_buffer_profile_attr_t
{
    
    SAI_BUFFER_PROFILE_ATTR_START,
    /** READ-WRITE */

    /** pointer to buffer pool object id [sai_object_id_t] (MANDATORY_ON_CREATE|CREATE_AND_SET)
    *  Pool id = SAI_NULL_OBJECT_ID can be used when profile is not associated with specific
    *  pool, for example for global port buffer. Not applicable to priority group or queue buffer profile */
    SAI_BUFFER_PROFILE_ATTR_POOL_ID = SAI_BUFFER_PROFILE_ATTR_START,

    /** reserved buffer size in bytes [sai_uint32_t] (MANDATORY_ON_CREATE|CREATE_AND_SET) */
    SAI_BUFFER_PROFILE_ATTR_BUFFER_SIZE,

    /** shared threshold mode for the buffer profile [sai_buffer_threadhold_mode_t] (CREATE_AND_SET)
     * If set, this overrides SAI_BUFFER_POOL_ATTR_TH_MODE.
     * If not set, use SAI_BUFFER_POOL_ATTR_TH_MODE. 
     * (default to value set in SAI_BUFFER_POOL_ATTR_TH_MODE.) */
    SAI_BUFFER_PROFILE_ATTR_TH_MODE,

    /** dynamic threshold for the shared usage [sai_int8_t]
     * The threshold is set to the 2^n of available buffer of the pool.
     * Mandatory when SAI_BUFFER_POOL_TH_MODE = SAI_BUFFER_THRESHOLD_MODE_DYNAMIC
     * (CREATE_AND_SET). */
    SAI_BUFFER_PROFILE_ATTR_SHARED_DYNAMIC_TH,

    /** static threshold for the shared usage in bytes [sai_uint32_t]
     * Mandatory when SAI_BUFFER_POOL_TH_MODE = SAI_BUFFER_THRESHOLD_MODE_STATIC
     * When set to zero there is no limit for the shared usage.
     * (CREATE_AND_SET) */
    SAI_BUFFER_PROFILE_ATTR_SHARED_STATIC_TH,

    /** set the buffer profile XOFF threshold in bytes [sai_uint32_t]
     * Valid only for ingress PG (CREATE_AND_SET).
     * Specifies the maximum available buffer for a PG after XOFF is 
     * generated (i.e. headroom buffer). Note that the available 
     * headroom buffer is dependent on XOFF_SIZE. If the user has 
     * set XOFF_SIZE = 0, the PG headroom buffer is equal to XOFF_TH
     * and it is not shared. If the user has set XOFF_SIZE > 0, the 
     * total headroom pool buffer for all PGs is equal to XOFF_SIZE
     * and XOFF_TH specifies the maximum amount of headroom pool 
     * buffer one PG can use.
     * default to 0. */
    SAI_BUFFER_PROFILE_ATTR_XOFF_TH,

    /** set the buffer profile XON non-hysteresis threshold in byte [sai_uint32_t]
     * Valid only for ingress PG (CREATE_AND_SET)
     * default to 0.
     * Generate XON when the total buffer usage of this PG is less than the maximum of XON_TH
     * and the total buffer limit minus XON_OFFSET_TH, and available buffer in the PG buffer
     * is larger than the XOFF_TH.
     * The XON trigger condition is governed by:
     *     total buffer usage <= max(XON_TH, total buffer limit - XON_OFFSET_TH)
     */
    SAI_BUFFER_PROFILE_ATTR_XON_TH,

    /** set the buffer profile XON hysteresis threshold in byte [sai_uint32_t]
     * Valid only for ingress PG (CREATE_AND_SET)
     * default to 0xffffffff.
     * Generate XON when the total buffer usage of this PG is less than the maximum of XON_TH
     * and the total buffer limit minus XON_OFFSET_TH, and available buffer in the PG buffer
     * is larger than the XOFF_TH.
     * The XON trigger condition is governed by:
     *     total buffer usage <= max(XON_TH, total buffer limit - XON_OFFSET_TH)
     */
    SAI_BUFFER_PROFILE_ATTR_XON_OFFSET_TH,

    SAI_BUFFER_PROFILE_ATTR_END,

} sai_buffer_profile_attr_t;

/**
 * @brief Create buffer profile
 * @param[out] buffer_profile_id buffer profile id
 * @param[in] attr_count number of attributes
 * @param[in] attr_list array of attributes
 * @return SAI_STATUS_SUCCESS on success
 *           Failure status code on error
 */
typedef sai_status_t(*sai_create_buffer_profile_fn)(
    _Out_ sai_object_id_t* buffer_profile_id,
    _In_ uint32_t attr_count,
    _In_ const sai_attribute_t *attr_list
    );

/**
 * @brief Remove buffer profile
 * @param[in] buffer_profile_id buffer profile id
 * @return SAI_STATUS_SUCCESS on success
 *           Failure status code on error
 */
typedef sai_status_t(*sai_remove_buffer_profile_fn)(
    _In_ sai_object_id_t buffer_profile_id
    );

/**
 * @brief Set buffer profile attribute
 * @param[in] buffer_profile_id buffer profile id
 * @param[in] attr attribute
 * @return SAI_STATUS_SUCCESS on success
 *           Failure status code on error
 */
typedef sai_status_t(*sai_set_buffer_profile_attr_fn)(
    _In_ sai_object_id_t buffer_profile_id,
    _In_ const sai_attribute_t *attr
    );

/**
 * @brief Get buffer profile attributes
 * @param[in] buffer_profile_id buffer profile id
 * @param[in] attr_count number of attributes
 * @param[inout] attr_list array of attributes
 * @return SAI_STATUS_SUCCESS on success
 *           Failure status code on error
 */
typedef sai_status_t(*sai_get_buffer_profile_attr_fn)(
    _In_ sai_object_id_t buffer_profile_id,
    _In_ uint32_t attr_count,
    _Inout_ sai_attribute_t *attr_list
    );

/**
 *  @brief buffer methods table retrieved with sai_api_query()
 */
typedef struct _sai_buffer_api_t
{
    sai_create_buffer_pool_fn                  create_buffer_pool;
    sai_remove_buffer_pool_fn                  remove_buffer_pool;
    sai_set_buffer_pool_attr_fn                set_buffer_pool_attr;
    sai_get_buffer_pool_attr_fn                get_buffer_pool_attr;
    sai_get_buffer_pool_stats_fn               get_buffer_pool_stats;
    sai_set_ingress_priority_group_attr_fn     set_ingress_priority_group_attr;
    sai_get_ingress_priority_group_attr_fn     get_ingress_priority_group_attr;
    sai_get_ingress_priority_group_stats_fn    get_ingress_priority_group_stats;
    sai_clear_ingress_priority_group_stats_fn  clear_ingress_priority_group_stats;
    sai_create_buffer_profile_fn               create_buffer_profile;
    sai_remove_buffer_profile_fn               remove_buffer_profile;
    sai_set_buffer_profile_attr_fn             set_buffer_profile_attr;
    sai_get_buffer_profile_attr_fn             get_buffer_profile_attr;
} sai_buffer_api_t;

/**
 *\}
 */

#endif // __SAIBUFFER_H_
